#!/usr/bin/python
#backups.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#

import os
import sys
import time
import random
import gc


try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in backups.py')

from twisted.internet.defer import Deferred, maybeDeferred
from twisted.internet import threads


import lib.dhnio as dhnio
import lib.misc as misc
import lib.nameurl as nameurl
import lib.transport_control as transport_control
import lib.settings as settings
import lib.contacts as contacts
import lib.eccmap as eccmap
import lib.tmpfile as tmpfile
import lib.diskspace as diskspace
from lib.automat import Automat


import backup_monitor
import backup_rebuilder  
import fire_hire
import list_files_orator
import lib.automats as automats

import raidread
import p2p_service
import io_throttle
import backup_db
import contacts_status

#------------------------------------------------------------------------------ 

_RemoteFiles = {}
_LocalFiles = {}
_RemoteMaxBlockNumbers = {}
_LocalMaxBlockNumbers = {}
_LocalBackupSize = {}
_BackupsInProcess = []
_SuppliersSet = None
_BackupStatusNotifyCallback = None
_StatusCallBackForGuiBackup = None
_LocalFilesNotifyCallback = None
_UpdatedBackupIDs = set()
_RepaintingTask = None   
_RepaintingTaskDelay = 0.1  

#------------------------------------------------------------------------------ 

def init():
    dhnio.Dprint(4, 'backups.init')
    RepaintingProcess(True)
    ReadLocalFiles()
    ReadLatestRawListFiles()


def shutdown():
    dhnio.Dprint(4, 'backups.shutdown')
    RepaintingProcess(False)

#------------------------------------------------------------------------------ 

# index for all remote files (on suppliers HDD's) stored in dictionary
# values are 0 or 1 - this mean file exist or not 
# all entries are indexed by [backupID][blockNum]['D' or 'P'][supplierNum]
def remote_files():
    global _RemoteFiles
    return _RemoteFiles


# max block number for every remote backup ID
def remote_max_block_numbers():
    global _RemoteMaxBlockNumbers
    return _RemoteMaxBlockNumbers


# index for all local files (on our HDD) stored in dictionary
# values are 0 or 1 - this mean file exist or not 
# all entries are indexed by [backupID][blockNum]['D' or 'P'][supplierNum]
def local_files():
    global _LocalFiles
    return _LocalFiles


# max block number for every local backup ID
def local_max_block_numbers():
    global _LocalMaxBlockNumbers
    return _LocalMaxBlockNumbers


def local_backup_size():
    global _LocalBackupSize
    return _LocalBackupSize


# currently working backups (started from dobackup.py)
def backups_in_process():
    global _BackupsInProcess
    return _BackupsInProcess


def suppliers_set():
    global _SuppliersSet
    if _SuppliersSet is None:
        _SuppliersSet = SuppliersSet(contacts.getSupplierIDs())
    return _SuppliersSet


#------------------------------------------------------------------------------ 
# this should represent the set of suppliers for either the user or for a customer the user
# is acting as a scrubber for
class SuppliersSet:
    def __init__(self, supplierList):
        self.suppliers = [] 
        self.supplierCount = 0 
        self.successPackets = {}
        self.failedPackets = {}
        self.failedSuppliers = set()
        self.UpdateSuppliers(supplierList)

    def GetActiveArray(self):
        activeArray = [0] * self.supplierCount
        for i in range(self.supplierCount):
            if contacts_status.isOnline(self.suppliers[i]):
                activeArray[i] = 1
            else:
                activeArray[i] = 0
        return activeArray

    def ChangedArray(self, supplierList):
        changedArray = [0] * self.supplierCount
        changedIdentities = []
        for i in range(self.supplierCount):
            if supplierList[i] != self.suppliers[i]:
                changedArray[i] = 1
                changedIdentities.append(supplierList[i])
        return changedArray, changedIdentities

    def SuppliersChanged(self, supplierList):
        if len(supplierList) != self.supplierCount:
            return True
        for i in range(self.supplierCount):
            if supplierList[i] != self.suppliers[i]:
                return True
        return False

    # if suppliers 1 and 3 changed, return [1,3]
    def SuppliersChangedNumbers(self, supplierList):
        changedList = []
        for i in range(self.supplierCount):
            if supplierList[i] != self.suppliers[i]:
                changedList.append(i)
        return changedList

    def SupplierCountChanged(self, supplierList):
        if len(supplierList) != self.supplierCount:
            return True
        else:
            return False

    def UpdateSuppliers(self, supplierList):
        self.suppliers = supplierList
        self.supplierCount = len(self.suppliers)
        self.successPackets.clear()
        self.failedPackets.clear()
        for supplierNum in range(self.supplierCount):
            self.successPackets[supplierNum] = 0
            self.failedPackets[supplierNum] = 0
        self.failedSuppliers.clear()

#------------------------------------------------------------------------------ 

def IncomingListFiles(supplierNum, packet, listFileText):
    supplierIdentity = packet.OwnerID
    #self.requestedListFilesPacketIDs.discard(supplierIdentity)
    #dhnio.Dprint(8, 'backups.IncomingListFiles for supplier %d other requests: %s' % (supplierNum, self.requestedListFilesPacketIDs))

    if supplierNum >= suppliers_set().supplierCount:
        dhnio.Dprint(2, 'backups.IncomingListFiles WARNING incorrect supplier number %d, number of suppliers is %d' % (supplierNum, suppliers_set().supplierCount))
        return

    backups2remove = ReadRawListFiles(supplierNum, listFileText)
    list_files_orator.IncommingListFiles(supplierNum, packet, listFileText)
    SaveLatestRawListFiles(supplierIdentity, listFileText)
    if len(backups2remove) > 0:
        p2p_service.RequestDeleteListBackups(backups2remove)


def ReadRawListFiles(supplierNum, listFileText):
    backups2remove = []
    # new code            
    # examples:
    # F20090709034221PM-0-Data from 0-1000
    # F20090709034221PM-0-Data from 0-1000 missing 1,3,
    for line in listFileText.splitlines():
        line = line.strip()
        # comment lines in Files reports start with blank,
        if line == '':
            continue
        # also don't consider the identity a backup,
        if line.find('http://') != -1 or line.find('.xml') != -1:
            continue
        # nor the backup_info.xml
        if line == settings.BackupInfoFileName():
            continue
        words = line.split(' ')
        # minimum is 3 words: "F20090709034221PM-0-Data", "from", "0-1000"
        if len(words) < 3:
            dhnio.Dprint(2, 'backups.ReadRawListFiles WARNING incorrect line:[%s]' % line)
            continue
        try:
            backupID, lineSupplierNum, dataORparity = words[0].split('-')
            minBlockNum, maxBlockNum = words[2].split('-')
            lineSupplierNum = int(lineSupplierNum)
            maxBlockNum = int(maxBlockNum)
        except:
            dhnio.Dprint(2, 'backups.ReadRawListFiles WARNING incorrect line:[%s]' % line)
            continue
        if backupID == '':
            dhnio.Dprint(2, 'backups.ReadRawListFiles WARNING incorrect line:[%s]' % line)
            continue
        if dataORparity not in ['Data', 'Parity']:
            dhnio.Dprint(2, 'backups.ReadRawListFiles WARNING incorrect line:[%s]' % line)
            continue
        if lineSupplierNum != supplierNum:
            # this mean supplier have old files and we do not need it 
            # TODO what to do...  remove the files? Send "DeleteFile" packet?
            backups2remove.append(backupID)
            continue
        if backup_db.IsDeleted(backupID):
            backups2remove.append(backupID)
            continue
        missingBlocksSet = set()
        if len(words) ==  5:
            if words[3].strip() != 'missing':
                dhnio.Dprint(2, 'backups.ReadRawListFiles WARNING incorrect line:[%s]' % line)
                continue
            missingBlocksSet = set(words[4].split(','))
        if not remote_files().has_key(backupID):
            remote_files()[backupID] = {}
            dhnio.Dprint(6, 'backups.ReadRawListFiles new remote entry for %s created in the memory' % backupID)
        # +1 because range(2) give us [0,1] but we want [0,1,2]
        for blockNum in range(maxBlockNum+1):
            if not remote_files()[backupID].has_key(blockNum):
                remote_files()[backupID][blockNum] = {
                    'D': [0] * suppliers_set().supplierCount,
                    'P': [0] * suppliers_set().supplierCount,}
            # we set -1 if the file is missing and 1 if exist, so 0 mean "no info yet" ... smart!
            bit = -1 if str(blockNum) in missingBlocksSet else 1 
            if dataORparity == 'Data':
                remote_files()[backupID][blockNum]['D'][supplierNum] = bit
            elif dataORparity == 'Parity':
                remote_files()[backupID][blockNum]['P'][supplierNum] = bit
        # save max block number for this backup
        if not remote_max_block_numbers().has_key(backupID):
            remote_max_block_numbers()[backupID] = -1 
        if maxBlockNum > remote_max_block_numbers()[backupID]:
            remote_max_block_numbers()[backupID] = maxBlockNum
        
        # mark this backup to be repainted
        RepaintBackup(backupID)
    # return list of backupID's which is too old but stored on suppliers machines 
    return backups2remove
            

def SaveLatestRawListFiles(idurl, listFileText):
    supplierPath = settings.SupplierPath(idurl)
    if not os.path.isdir(supplierPath):
        try:
            os.makedirs(supplierPath)
        except:
            dhnio.DprintException()
    dhnio.WriteFile(settings.SupplierListFilesFilename(idurl), listFileText)


def ReadLatestRawListFiles():
    dhnio.Dprint(4, 'backups.ReadLatestRawListFiles')
    for idurl in contacts.getSupplierIDs():
        filename = os.path.join(settings.SupplierPath(idurl, 'listfiles'))
        if os.path.isfile(filename):
            listFileText = dhnio.ReadTextFile(filename)
            if listFileText.strip() != '':
                ReadRawListFiles(contacts.numberForSupplier(idurl), listFileText)


def ScanMissingBlocks(backupID):
    missingBlocks = set()
    localMaxBlockNum = local_max_block_numbers().get(backupID, -1)
    remoteMaxBlockNum = remote_max_block_numbers().get(backupID, -1)
    supplierActiveArray = suppliers_set().GetActiveArray()
    #dhnio.Dprint(6, 'backups.ScanMissingBlocks supplierActiveArray=%s' % supplierActiveArray)

    if not remote_files().has_key(backupID):
        if not local_files().has_key(backupID):
            # we have no local and no remote info for this backup
            # no chance to do some rebuilds...
            # TODO but how we get here ?! 
            dhnio.Dprint(4, 'backups.ScanMissingBlocks no local and no remote info for %s' % backupID)
        else:
            # we have no remote info, but some local files exists
            # so let's try to sent all of them
            # need to scan all block numbers 
            for blockNum in range(localMaxBlockNum):
                # we check for Data and Parity packets
                localData = GetLocalDataArray(backupID, blockNum)
                localParity = GetLocalParityArray(backupID, blockNum)  
                for supplierNum in range(len(supplierActiveArray)):
                    # if supplier is not alive we can not send to him
                    # so no need to scan for missing blocks 
                    if supplierActiveArray[supplierNum] != 1:
                        continue
                    if localData[supplierNum] == 1:
                        missingBlocks.add(blockNum)
                    if localParity[supplierNum] == 1:
                        missingBlocks.add(blockNum)
    else:
        # now we have some remote info
        # we take max block number from local and remote
        maxBlockNum = max(remoteMaxBlockNum, localMaxBlockNum)
        # dhnio.Dprint(6, 'backups.ScanMissingBlocks maxBlockNum=%d' % maxBlockNum)
        # and increase by one because range(3) give us [0, 1, 2], but we want [0, 1, 2, 3]
        for blockNum in range(maxBlockNum + 1):
            # if we have few remote files, but many locals - we want to send all missed 
            if not remote_files()[backupID].has_key(blockNum):
                missingBlocks.add(blockNum)
                continue
            # take remote info for this block
            remoteData = GetRemoteDataArray(backupID, blockNum)
            remoteParity = GetRemoteParityArray(backupID, blockNum)  
            # now check every our supplier for every block
            for supplierNum in range(len(supplierActiveArray)):
                # if supplier is not alive we can not send to him
                # so no need to scan for missing blocks 
                if supplierActiveArray[supplierNum] != 1:
                    continue
                if remoteData[supplierNum] != 1:    # -1 means missing
                    missingBlocks.add(blockNum)     # 0 - no info yet
                if remoteParity[supplierNum] != 1:  # 1 - file exist on remote supplier 
                    missingBlocks.add(blockNum)
                
    # dhnio.Dprint(6, 'backups.ScanMissingBlocks %s' % missingBlocks)
    return list(missingBlocks)


def ReadLocalFiles():
    global _LocalFilesNotifyCallback
    local_files().clear()
    local_max_block_numbers().clear()
    local_backup_size().clear()
    counter = 0
    # for filename in os.listdir(tmpfile.subdir('data-par')):
    for filename in os.listdir(settings.getLocalBackupsDir()):
        if filename.startswith('newblock-'):
            continue
        try:
            backupID, blockNum, supplierNum, dataORparity  = filename.split('-')
            blockNum = int(blockNum)
            supplierNum = int(supplierNum)
        except:
            dhnio.Dprint(4, 'backups.ReadLocalFiles WARNING incorrect filename ' + filename)
            continue
        if dataORparity not in ['Data', 'Parity']:
            dhnio.Dprint(4, 'backups.ReadLocalFiles WARNING Data or Parity? ' + filename)
            continue
        if supplierNum >= suppliers_set().supplierCount:
            dhnio.Dprint(4, 'backups.ReadLocalFiles WARNING supplier number %d > %d %s' % (supplierNum, suppliers_set().supplierCount, filename))
            continue
        if not local_files().has_key(backupID):
            local_files()[backupID] = {}
            dhnio.Dprint(6, 'backups.ReadLocalFiles new local entry for %s created in the memory' % backupID)
        if not local_files()[backupID].has_key(blockNum):
            local_files()[backupID][blockNum] = {
                'D': [0] * suppliers_set().supplierCount,
                'P': [0] * suppliers_set().supplierCount}
        local_files()[backupID][blockNum][dataORparity[0]][supplierNum] = 1
        if not local_max_block_numbers().has_key(backupID):
            local_max_block_numbers()[backupID] = -1
        if local_max_block_numbers()[backupID] < blockNum:
            local_max_block_numbers()[backupID] = blockNum
        if not local_backup_size().has_key(backupID):
            local_backup_size()[backupID] = 0
        # local_backup_size()[backupID] += os.path.getsize(os.path.join(tmpfile.subdir('data-par'), filename))
        try:
            local_backup_size()[backupID] += os.path.getsize(os.path.join(settings.getLocalBackupsDir(), filename))
        except:
            dhnio.DprintException()
        counter += 1
    dhnio.Dprint(6, 'backups.ReadLocalFiles  %d files indexed' % counter)
    if dhnio.Debug(6):
        try:
            if sys.version_info >= (2, 6):
                #localSZ = sys.getsizeof(local_files())
                #remoteSZ = sys.getsizeof(remote_files())
                import lib.getsizeof
                localSZ = lib.getsizeof.total_size(local_files())
                remoteSZ = lib.getsizeof.total_size(remote_files())
                dhnio.Dprint(6, '    all local info uses %d bytes in the memory' % localSZ)
                dhnio.Dprint(6, '    all remote info uses %d bytes in the memory' % remoteSZ)
        except:
            dhnio.DprintException()
    if _LocalFilesNotifyCallback is not None:
        _LocalFilesNotifyCallback()


def SetSupplierList(supplierList):
    # going from 2 to 4 suppliers (or whatever) invalidates all backups
    # all suppliers was changed because its number was changed
    # so we lost everything!
    if len(supplierList) != suppliers_set().supplierCount:
        dhnio.Dprint(2, "backups.SetSupplierList got list of %d suppliers, but we have %d now!" % (len(supplierList), suppliers_set().supplierCount))
        # erase all remove info
        #remote_files() = {}
        remote_files().clear()
        remote_max_block_numbers().clear()
        # also erase local info
        #local_files() = {}
        local_files().clear()
        local_max_block_numbers().clear()
        local_backup_size().clear()
        # new ecc map will be taken later
        #self.eccMap = None
        # remove all local files and all backups
        DeleteAllBackups()
        # restart backup_monitor
        backup_monitor.Restart()
    # only single suppliers changed
    # need to erase info only for them 
    elif suppliers_set().SuppliersChanged(supplierList):
        # take a list of suppliers positions that was changed
        changedSupplierNums = suppliers_set().SuppliersChangedNumbers(supplierList)
        # notify io_throttle that we do not neeed already this suppliers
        for supplierNum in changedSupplierNums:
            dhnio.Dprint(2, "backups.SetSupplierList supplier %d changed: [%s]->[%s]" % (
                supplierNum, nameurl.GetName(suppliers_set().suppliers[supplierNum]), nameurl.GetName(supplierList[supplierNum])))
            io_throttle.DeleteSuppliers([suppliers_set().suppliers[supplierNum]])
        # remove remote info for this guys
        for backupID in remote_files().keys():
            for blockNum in remote_files()[backupID].keys():
                for supplierNum in changedSupplierNums:
                    remote_files()[backupID][blockNum]['D'][supplierNum] = 0
                    remote_files()[backupID][blockNum]['P'][supplierNum] = 0
        # restart backup_monitor
        backup_monitor.Restart()
    # finally save the list of current suppliers and clear all stats 
    suppliers_set().UpdateSuppliers(supplierList)


def RebuildReport(backupID, blockNum, remoteData, remoteParity, sentReports):
    if not remote_files().has_key(backupID):
        remote_files()[backupID] = {}
        dhnio.Dprint(6, 'backups.RebuildReport new remote entry for %s created in the memory' % backupID)
    if not remote_files()[backupID].has_key(blockNum):
        remote_files()[backupID][blockNum] = {
            'D': [0] * suppliers_set().supplierCount,
            'P': [0] * suppliers_set().supplierCount,}
    # save reconstructed block info into remote info structure, synchronize
    for supplierNum in range(suppliers_set().supplierCount):
        remote_files()[backupID][blockNum]['D'][supplierNum] = remoteData[supplierNum]
        remote_files()[backupID][blockNum]['P'][supplierNum] = remoteParity[supplierNum]
    # if we know only 5 blocks stored on remote machine
    # but we got reconstructed 6th block - remember this  
    remote_max_block_numbers()[backupID] = max(
        remote_max_block_numbers().get(backupID, -1),
        blockNum)
    
#    # remember sent results
#    for packetID, result in sentReports.items():
#        try:
#            backupID, blockNum, supplierNum, dataORparity = packetID.split("-")
#            blockNum = int(blockNum)
#            supplierNum = int(supplierNum)
#            supplierIDURL = suppliers_set().suppliers[supplierNum]
#            if result is 'acked':
#                suppliers_set().successPackets[supplierNum] += 1
#            else:
#                suppliers_set().failedPackets[supplierNum] += 1
#                if suppliers_set().failedPackets[supplierNum] >= 6:
#                    supplierIdentity = contacts.getContact(supplierIDURL)
#                    supplierContact = '' if supplierIdentity is None else supplierIdentity.getContact() 
#                    if not supplierContact.startswith('cspace://'):
#                        # mark this guy as failed if 6 times we were failed to sent to him
#                        suppliers_set().failedSuppliers.add(supplierIDURL)
#                        dhnio.Dprint(4, 'backups.RebuildReport %s have %d fails - mark him as failed' % (
#                            nameurl.GetName(supplierIDURL), suppliers_set().failedPackets[supplierNum]))
#                    else:
#                        # but if he is 'cspace' user - give him more chances - 24 times
#                        if suppliers_set().failedPackets[supplierNum] >= 24:
#                            suppliers_set().failedSuppliers.add(supplierIDURL)
#                            dhnio.Dprint(4, 'backups.RebuildReport %s is "cspace user" and have %d fails - mark him as failed' % (
#                                nameurl.GetName(supplierIDURL), suppliers_set().failedPackets[supplierNum]))
#                        
#        except:
#            dhnio.DprintException()

    # mark to repaint this backup in gui
    RepaintBackup(backupID)


def RepaintBackup(backupID): 
    global _UpdatedBackupIDs
    _UpdatedBackupIDs.add(backupID)


def RepaintingProcess(on_off):
    global _UpdatedBackupIDs
    global _BackupStatusNotifyCallback
    global _RepaintingTask
    global _RepaintingTaskDelay
    if on_off is False:
        _RepaintingTaskDelay = 0.5
        if _RepaintingTask is not None:
            if not _RepaintingTask.called:
                _RepaintingTask.cancel()
                _RepaintingTask = None
                _UpdatedBackupIDs.clear()
                return
    if len(_UpdatedBackupIDs) > 0:
        for backupID in _UpdatedBackupIDs:
            if _BackupStatusNotifyCallback is not None:
                _BackupStatusNotifyCallback(backupID)
        _UpdatedBackupIDs.clear()
        _RepaintingTaskDelay = 0.5
    else:
        if _RepaintingTaskDelay < 2.0:
            _RepaintingTaskDelay *= 2.0
    # attenuation
    _RepaintingTask = reactor.callLater(_RepaintingTaskDelay, RepaintingProcess, True)


# if there is a backup in process, one supplier may list more files in the backup than another,
# leading the backups to try to fix a backup in process.  We need to tell the backups
# when a backup is in process and when it is finished
def AddBackupInProcess(BackupName):
    backups_in_process().append(BackupName)


def RemoveBackupInProcess(BackupName):
    if BackupName in backups_in_process():
        backups_in_process().remove(BackupName)


def IsBackupInProcess(BackupID):
    return BackupID in backups_in_process()

#------------------------------------------------------------------------------ 
          
def DeleteAllBackups():
    for backupId in set(local_files().keys() + remote_files().keys()):
        DeleteBackup(backupId)
    DeleteAllLocalBackups()


# if the user deletes a backup, make sure we remove any work we're doing on it
def DeleteBackup(backupID, removeLocalFilesToo=True):
    dhnio.Dprint(4, 'backups.DeleteBackup ' + backupID)
    # abort backup if it just started and is running at the moment
    backup_db.AbortRunningBackup(backupID)
    # also remove from list of running backups  
    if backupID in backups_in_process():
        backups_in_process().remove(backupID)
    # if we have several copies of same ID in working queue - remove them all
    backup_rebuilder.RemoveBackupToWork(backupID)
    # if we requested for files for this backup - we do not need it anymore
    io_throttle.DeleteBackupRequests(backupID)
    # remove interests in transport_control
    transport_control.DeleteBackupInterest(backupID)
    # mark it as being deleted in the db
    backup_db.DeleteDirBackup(backupID)
    backup_db.Save()
    # finally remove local files for this backupID
    if removeLocalFilesToo:
        DeleteLocalBackupFiles(backupID)
    # remove all remote info for this backup from the memory 
    if remote_files().has_key(backupID):
        #del remote_files()[backupID]
        remote_files().pop(backupID)
        dhnio.Dprint(4, '  remote info for %s were removed from memory, other remote files:' % backupID)
        dhnio.Dprint(4, '  %s' % str(remote_files().keys()))
    if remote_max_block_numbers().has_key(backupID):
        del remote_max_block_numbers()[backupID]
    # also remove local info
    if local_files().has_key(backupID):
        #del local_files()[backupID]
        local_files().pop(backupID)
        dhnio.Dprint(4, '  local info for %s were removed from memory, other local files:' % backupID)
        dhnio.Dprint(4, '  %s' % str(local_files().keys()))
    if local_max_block_numbers().has_key(backupID):
        del local_max_block_numbers()[backupID]
    if local_backup_size().has_key(backupID):
        del local_backup_size()[backupID]


def DeleteAllSupplierData(supplierNum):
    io_throttle.DeleteSuppliers([suppliers_set().suppliers[supplierNum]])


def DeleteLocalBackupFiles(backupID):
    dhnio.Dprint(4, 'backups.DeleteLocalBackupFiles ' + backupID)
    num = 0
    sz = 0
    # localDir = tmpfile.subdir('data-par')
    localDir = settings.getLocalBackupsDir()
#    if backupID not in local_files():
#        local_files()[backupID] = {}
    for filename in os.listdir(localDir):
        if filename.startswith(backupID):
#            try:
#                backupID_, blockNum, supplierNum, dataORparity = filename.split("-")
#                blockNum = int(blockNum)
#                supplierNum = int(supplierNum)
#            except:
#                dhnio.DprintException()
#                continue
#            if backupID_ != backupID:
#                dhnio.Dprint(4, 'backups.DeleteLocalBackupFiles WARNING incorrect file name: %s' % filename)
#                continue
            filepath = os.path.join(localDir, filename)
            try:
                sz += os.path.getsize(filepath) 
                os.remove(filepath)
                num += 1
#                if dataORparity == 'Data':
#                    local_files()[backupID][blockNum]['D'][supplierNum] = 0
#                elif dataORparity == 'Parity':
#                    local_files()[backupID][blockNum]['P'][supplierNum] = 0
#                else:
#                    dhnio.Dprint(4, 'backups.DeleteLocalBackupFiles WARNING incorrect file name: %s' % filename)
            except:
                dhnio.Dprint(2, 'backups.DeleteLocalBackupFiles ERROR can not remove ' + filepath)
                dhnio.DprintException()
    local_files()[backupID] = []
    local_max_block_numbers()[backupID] = -1
    local_backup_size()[backupID] = 0
    return num, sz


def DeleteAllLocalBackups():
    dhnio.Dprint(4, 'backups.DeleteAllLocalBackups')
    num = 0
    sz = 0
    backupIDs = set()
    # localDir = tmpfile.subdir('data-par')
    localDir = settings.getLocalBackupsDir()
    for filename in os.listdir(localDir):
        filepath = os.path.join(localDir, filename)
        try:
            backupID, blockNum, supplierNum, dataORparity = filename.split("-")
            sz += os.path.getsize(filepath) 
            os.remove(filepath)
            num += 1
            backupIDs.add(backupID)
        except:
            dhnio.Dprint(1, 'backups.DeleteAllLocalBackups ERROR can not remove ' + filepath)
            dhnio.DprintException()
    dhnio.Dprint(4, 'backups.DeleteAllLocalBackups %d backups removed in %d files of total size %s' % (
        len(backupIDs), num, diskspace.MakeStringFromBytes(sz)))

#------------------------------------------------------------------------------ 

def GetListFilesBackupIds(supplierNum, supplierId, listFileText):
    rawlist = listFileText.splitlines()
    resultlist = []
    for line in rawlist:
        # comment lines in Files reports start with blank,
        # also don't consider the identity a backup,
        # nor the backup_info.xml
        if len(line) > 0 and (line[0] == ' ' or line.strip() == supplierId or line == settings.BackupInfoFileName()):
            continue
        dashindex = line.find("-")
        backupId = ''
        if dashindex>0: 
            # if this is supplier 3 make sure we have a -3-
            if line.find("-" + str(supplierNum) + "-") > 0:   
                # if there is a dash use only part before the dash
                backupId=line[0:dashindex]                    
        if backupId != '' and backupId not in resultlist:
            resultlist.append(backupId)
    return misc.sorted_backup_ids(resultlist)


# if we get a line "F20090709034221PM-0-Data from 0-1000"
# or "F20090709034221PM-0-Data from 0-1000 missing 1,3,"
# this will return 1000
def GetLineMaxBlockNum(line):
    try:
        maxBlock = -1
        workLine = line
        if line.find(" missing ") > 0:
            workLine = line[0:line.find(" missing ")]
        lineMax = int(workLine[workLine.rfind("-")+1:])
        if lineMax > maxBlock:
            maxBlock = lineMax
        return maxBlock
    except:
        return -1


def GetLineMissingBlocks(line, lineMax, backupMax):
    missingArray = []
    missingindex = line.find(" missing ") 
    if missingindex != -1:
        missingArray = line[missingindex+9:].split(",")
    if '' in missingArray:
        missingArray.remove('')
    if backupMax > lineMax:
        intMissingArray = range(lineMax+1,backupMax+1)
        for i in intMissingArray:
            missingArray.append(str(i))
    return missingArray


def GetBackupStats(backupID):
    if not remote_files().has_key(backupID):
        return 0, 0, [(0, 0)] * suppliers_set().supplierCount
    percentPerSupplier = 100.0 / suppliers_set().supplierCount
    # ??? maxBlockNum = remote_max_block_numbers().get(backupID, -1)
    maxBlockNum = GetKnownMaxBlockNum(backupID)
    fileNumbers = [0] * suppliers_set().supplierCount
    totalNumberOfFiles = 0
    for blockNum in remote_files()[backupID].keys():
        for supplierNum in range(len(fileNumbers)):
            if supplierNum < suppliers_set().supplierCount:
                if remote_files()[backupID][blockNum]['D'][supplierNum] == 1:
                    fileNumbers[supplierNum] += 1
                    totalNumberOfFiles += 1
                if remote_files()[backupID][blockNum]['P'][supplierNum] == 1:
                    fileNumbers[supplierNum] += 1
                    totalNumberOfFiles += 1
    statsArray = []
    for supplierNum in range(suppliers_set().supplierCount):
        if maxBlockNum > -1:
            # 0.5 because we count both Parity and Data.
            percent = percentPerSupplier * 0.5 * fileNumbers[supplierNum] / ( maxBlockNum + 1 )
        else:
            percent = 0.0
        statsArray.append(( percent, fileNumbers[supplierNum] ))
    del fileNumbers 
    return totalNumberOfFiles, maxBlockNum, statsArray

# return totalPercent, totalNumberOfFiles, totalSize, maxBlockNum, statsArray
def GetBackupLocalStats(backupID):
    # ??? maxBlockNum = local_max_block_numbers().get(backupID, -1)
    maxBlockNum = GetKnownMaxBlockNum(backupID)
    if not local_files().has_key(backupID):
        return 0, 0, 0, maxBlockNum, [(0, 0)] * suppliers_set().supplierCount
    percentPerSupplier = 100.0 / suppliers_set().supplierCount
    totalNumberOfFiles = 0
    fileNumbers = [0] * suppliers_set().supplierCount
    for blockNum in range(maxBlockNum + 1):
        if blockNum not in local_files()[backupID].keys():
            continue
#    for blockNum in local_files()[backupID].keys():
        for supplierNum in range(len(fileNumbers)):
            if supplierNum < suppliers_set().supplierCount:
                if local_files()[backupID][blockNum]['D'][supplierNum] == 1:
                    fileNumbers[supplierNum] += 1
                    totalNumberOfFiles += 1
                if local_files()[backupID][blockNum]['P'][supplierNum] == 1:
                    fileNumbers[supplierNum] += 1
                    totalNumberOfFiles += 1
    statsArray = []
    for supplierNum in range(suppliers_set().supplierCount):
        if maxBlockNum > -1:
            # 0.5 because we count both Parity and Data.
            percent = percentPerSupplier * 0.5 * fileNumbers[supplierNum] / ( maxBlockNum + 1 )
        else:
            percent = 0.0
        statsArray.append(( percent, fileNumbers[supplierNum] ))
    del fileNumbers 
    totalPercent = 100.0 * 0.5 * totalNumberOfFiles / ((maxBlockNum + 1) * suppliers_set().supplierCount)
    return totalPercent, totalNumberOfFiles, local_backup_size().get(backupID, 0), maxBlockNum, statsArray


def GetBackupBlocksAndPercent(backupID):
    if not remote_files().has_key(backupID):
        return 0, 0
    # get max block number
    # ??? maxBlockNum = remote_max_block_numbers().get(backupID, -1)
    maxBlockNum = GetKnownMaxBlockNum(backupID)
    if maxBlockNum == -1:
        return 0, 0
    # we count all remote files for this backup
    fileCounter = 0
    for blockNum in remote_files()[backupID].keys():
        for supplierNum in range(suppliers_set().supplierCount):
            if remote_files()[backupID][blockNum]['D'][supplierNum] == 1:
                fileCounter += 1
            if remote_files()[backupID][blockNum]['P'][supplierNum] == 1:
                fileCounter += 1
    # +1 since zero based and *0.5 because Data and Parity
    return maxBlockNum + 1, 100.0 * 0.5 * fileCounter / ((maxBlockNum + 1) * suppliers_set().supplierCount)

# return : blocks, percent, weak block, weak percent
def GetBackupRemoteStats(backupID, only_available_files=True):
    if not remote_files().has_key(backupID):
        return 0, 0, 0, 0
    # get max block number
    # ??? maxBlockNum = remote_max_block_numbers().get(backupID, -1)
    maxBlockNum = GetKnownMaxBlockNum(backupID)
    if maxBlockNum == -1:
        return 0, 0, 0, 0
    supplierCount = suppliers_set().supplierCount
    fileCounter = 0
    weakBlockNum = -1
    lessSuppliers = supplierCount
    activeArray = suppliers_set().GetActiveArray()
    # we count all remote files for this backup - scan all blocks
    for blockNum in range(maxBlockNum+1):
        if blockNum not in remote_files()[backupID].keys():
            lessSuppliers = 0
            weakBlockNum = blockNum
            continue
        goodSuppliers = supplierCount
        for supplierNum in range(supplierCount):
            if activeArray[supplierNum] != 1 and only_available_files:
                goodSuppliers -= 1
                continue
            if remote_files()[backupID][blockNum]['D'][supplierNum] != 1 or remote_files()[backupID][blockNum]['P'][supplierNum] != 1:
                goodSuppliers -= 1
            if remote_files()[backupID][blockNum]['D'][supplierNum] == 1:
                fileCounter += 1
            if remote_files()[backupID][blockNum]['P'][supplierNum] == 1:
                fileCounter += 1
        if goodSuppliers < lessSuppliers:
            lessSuppliers = goodSuppliers
            weakBlockNum = blockNum
    # +1 since zero based and *0.5 because Data and Parity
    return (maxBlockNum + 1, 100.0 * 0.5 * fileCounter / ((maxBlockNum + 1) * supplierCount),
            weakBlockNum, 100.0 * float(lessSuppliers) / float(supplierCount))


def GetBackupIds():
    return misc.sorted_backup_ids(remote_files().keys())


def GetKnownMaxBlockNum(backupID):
    return max(remote_max_block_numbers().get(backupID, -1), 
               local_max_block_numbers().get(backupID, -1))


def GetLocalDataArray(backupID, blockNum):
    if not local_files().has_key(backupID):
        return [0] * suppliers_set().supplierCount
    if not local_files()[backupID].has_key(blockNum):
        return [0] * suppliers_set().supplierCount
    return local_files()[backupID][blockNum]['D']


def GetLocalParityArray(backupID, blockNum):
    if not local_files().has_key(backupID):
        return [0] * suppliers_set().supplierCount
    if not local_files()[backupID].has_key(blockNum):
        return [0] * suppliers_set().supplierCount
    return local_files()[backupID][blockNum]['P']
    

def GetRemoteDataArray(backupID, blockNum):
    if not remote_files().has_key(backupID):
        return [0] * suppliers_set().supplierCount
    if not remote_files()[backupID].has_key(blockNum):
        return [0] * suppliers_set().supplierCount
    return remote_files()[backupID][blockNum]['D']

    
def GetRemoteParityArray(backupID, blockNum):
    if not remote_files().has_key(backupID):
        return [0] * suppliers_set().supplierCount
    if not remote_files()[backupID].has_key(blockNum):
        return [0] * suppliers_set().supplierCount
    return remote_files()[backupID][blockNum]['P']


def GetSupplierStats(supplierNum):
    result = {}
    files = total = 0
    for backupID in remote_files().keys():
        result[backupID] = [0, 0]
        for blockNum in remote_files()[backupID].keys():
            if remote_files()[backupID][blockNum]['D'][supplierNum] == 1:
                result[backupID][0] += 1
                files += 1 
            if remote_files()[backupID][blockNum]['P'][supplierNum] == 1:
                result[backupID][0] += 1
                files += 1
            result[backupID][1] += 2
            total += 2
    return files, total, result

#def GetSupplierLocalFiles(supplierNum):
#    result = []
#    files = total = 0
#    for backupID in local_files().keys():
#        for blockNum

def GetFailedSuppliers():
    return list(suppliers_set().failedSuppliers)

def GetSuppliersFails():
    # we try to send to this guys and we sure they are online
    # but somehow our packets was not sent correctly
    # so we remember people who have failed packets - failed suppliers
    result = {}
    for supplierNum in range(suppliers_set().supplierCount):
        idurl = suppliers_set().suppliers[supplierNum]
        result[idurl] = suppliers_set().failedPackets[supplierNum]
    dhnio.Dprint(6, 'backups.GetSuppliersFails ' + str(result))
    return result


def GetWeakLocalBlock(backupID):
    # scan all local blocks for this backup 
    # and find the worst block 
    supplierCount = suppliers_set().supplierCount
    if not local_files().has_key(backupID):
        return -1, 0, supplierCount
    maxBlockNum = GetKnownMaxBlockNum(backupID)
    weakBlockNum = -1
    lessSuppliers = supplierCount
    for blockNum in range(maxBlockNum+1):
        if blockNum not in local_files()[backupID].keys():
            return blockNum, 0, supplierCount
        goodSuppliers = supplierCount
        for supplierNum in range(supplierCount):
            if  local_files()[backupID][blockNum]['D'][supplierNum] != 1 or local_files()[backupID][blockNum]['P'][supplierNum] != 1:
                goodSuppliers -= 1
        if goodSuppliers < lessSuppliers:
            lessSuppliers = goodSuppliers
            weakBlockNum = blockNum
    return weakBlockNum, lessSuppliers, supplierCount
 
    
def GetWeakRemoteBlock(backupID):
    # scan all remote blocks for this backup 
    # and find the worst block - less suppliers keeps the data and stay online 
    supplierCount = suppliers_set().supplierCount
    if not remote_files().has_key(backupID):
        return -1, 0, supplierCount
    maxBlockNum = GetKnownMaxBlockNum(backupID)
    weakBlockNum = -1
    lessSuppliers = supplierCount
    activeArray = suppliers_set().GetActiveArray()
    for blockNum in range(maxBlockNum+1):
        if blockNum not in remote_files()[backupID].keys():
            return blockNum, 0, supplierCount
        goodSuppliers = supplierCount
        for supplierNum in range(supplierCount):
            if activeArray[supplierNum] != 1:
                goodSuppliers -= 1
                continue
            if  remote_files()[backupID][blockNum]['D'][supplierNum] != 1 or remote_files()[backupID][blockNum]['P'][supplierNum] != 1:
                goodSuppliers -= 1
        if goodSuppliers < lessSuppliers:
            lessSuppliers = goodSuppliers
            weakBlockNum = blockNum
    return weakBlockNum, lessSuppliers, supplierCount

#------------------------------------------------------------------------------ 

def SetBackupStatusNotifyCallback(callBack):
    global _BackupStatusNotifyCallback
    _BackupStatusNotifyCallback = callBack

def SetLocalFilesNotifyCallback(callback):
    global _LocalFilesNotifyCallback
    _LocalFilesNotifyCallback = callback

#def SetStatusCallBackForGuiBackup(callBack):
#    global _StatusCallBackForGuiBackup
#    StatusCallBackForGuiBackup = callBack

#------------------------------------------------------------------------------ 


if __name__ == "__main__":
    init()
    import pprint
    pprint.pprint(GetBackupIds())














#!/usr/bin/python
#identitycache.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
# identitycache.py stores local copies of identities
#
# This fetches identities off the web and stores an xml copy in file and an identity object
#   in a dictionary.  
# Other parts of DHN call this to get an identity using an IDURL.

import os

from twisted.internet.defer import Deferred

import dhnio
import dhnnet
import settings
import identitydb

_CachingTasks = {}

#-------------------------------------------------------------------------------

def init(success_func=None, fail_func=None):
    dhnio.Dprint(4, 'identitycache.init')
    identitydb.init()
    CacheCentralID(success_func, fail_func)

def CacheCentralID(success_func=None, fail_func=None):
    dhnio.Dprint(6, 'identitycache.CacheCentralID')
    if HasKey(settings.CentralID()):
        if success_func:
            success_func('')
        return
    src = dhnio._read_data(os.path.join(dhnio.getExecutableDir(), 'dhncentral.xml'))
    if src:
        if identitydb.update(settings.CentralID(), src):
            if success_func:
                success_func('')
            return
    d = immediatelyCaching(settings.CentralID())
    if success_func is not None:
        d.addCallback(success_func)
    if fail_func is not None:
        d.addErrback(fail_func)

def Clear(excludeList=None):
    identitydb.clear(excludeList)

def CacheLen():
    return identitydb.size()

def PrintID(url):
    identitydb.print_id(url)

def PrintCacheKeys():
    identitydb.print_keys()

def PrintAllInCache():
    identitydb.print_cache()

def HasKey(url):
    return identitydb.has_key(url)

def FromCache(url):
    return identitydb.get(url)

def GetIDURLsByContact(contact):
    return identitydb.get_idurls_by_contact(contact)

def GetContacts(url):
    return identitydb.idcontacts(url)

def Remove(url):
    return identitydb.remove(url)

def UpdateAfterChecking(url, xml_src):
    #dhnio.Dprint(12, 'identitycache.UpdateAfterChecking ' + url)
    return identitydb.update(url, xml_src)

#------------------------------------------------------------------------------ 

def getPageSuccess(src, url):
    UpdateAfterChecking(url, src)
    return src

def getPageFail(x, url):
    dhnio.Dprint(6, "identitycache.getPageFail NETERROR in request to " + url)
    return x

def pageRequestTwisted(url):
    dhnio.Dprint(12, 'identitycache.pageRequestTwisted ' + url)
    d = dhnnet.getPageTwisted(url)
    d.addCallback(getPageSuccess, url)
    d.addErrback(getPageFail, url)
    return d

# Even if we have a copy in cache we are to try and read another one
def scheduleForCaching(url):
    return pageRequestTwisted(url)

#------------------------------------------------------------------------------ 

def immediatelyCaching(url, success_func=None, fail_func=None):
    global _CachingTasks
    if _CachingTasks.has_key(url):
        return _CachingTasks[url]
    
    def _getPageSuccess(src, url, res):
        global _CachingTasks
        _CachingTasks.pop(url)
        if UpdateAfterChecking(url, src):
            if success_func is not None:
                success_func(url+'\n'+src)
            res.callback(src)
        else:
            if fail_func is not None:
                fail_func(url+'\n'+src)
            res.errback(src)
        
    def _getPageFail(x, url, res):
        global _CachingTasks
        _CachingTasks.pop(url)
        if fail_func is not None:
            fail_func(x)
        res.errback(x)
        
    result = Deferred()
    d = dhnnet.getPageTwisted(url)
    d.addCallback(_getPageSuccess, url, result)
    d.addErrback(_getPageFail, url, result)
    
    _CachingTasks[url] = result
    
    return result

#------------------------------------------------------------------------------ 

def SetLocalIPs(local_ips):
    return identitydb.set_local_ips_dict(local_ips)

def GetLocalIP(idurl):
    return identitydb.get_local_ip(idurl)

def HasLocalIP(idurl):
    return identitydb.has_local_ip(idurl)

#------------------------------------------------------------------------------ 





#!/usr/bin/python
#dhnpacket.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
# These packets usually hold on the order of 1 MB.
# Something equal to a packet number so we can detect duplicates in transport
from twisted.internet import threads
from twisted.internet.defer import Deferred


import types
import datetime


import dhnio
import commands
import misc
import dhncrypto
import packetid
import contacts
#import signer


# Legal Commands are in commands.py
# Packet Fields are all strings (no integers, objects, etc):
class dhnpacket:
# PacketID - string of the above 4 "Number"s with "-" separator to uniquely identify a packet
# on the local machine.  Can be used for filenames, and to prevent duplicates.
    def __init__(self, Command, OwnerID, CreatorID, PacketID, Payload, RemoteID,):# SignInThread=False):
        self.Command = Command               
        # who owns this data and pays bills - http://cate.com/id1.xml
        self.OwnerID = OwnerID                
        # signer - http://cate.com/id1.xml - might be an authorized scrubber
        self.CreatorID = CreatorID            
        # functions for making and reading parts of this
        self.PacketID = PacketID              
        self.Date = datetime.datetime.now().strftime("%Y/%m/%d %I:%M:%S %p")
        # main body of binary data
        self.Payload = Payload                
        # want full IDURL for other party so troublemaker could not
        # use his packets to mess up other nodes by sending it to them
        self.RemoteID = RemoteID             
        self.Signature=None

        SignInThread = False
        # signature on Hash is always by CreatorID
        if not SignInThread:
            self.SigningDefer = None
            self.Sign()
        else:
            self.SigningDefer = Deferred()
            self.SignInThread()

    def __repr__(self):
        return 'dhnpacket (command=%s id=%s)' % (str(self.Command), str(self.PacketID))

    def Sign(self):
        self.Signature = self.GenerateSignature()  # usually just done at packet creation
        return self

    def SignInThread(self):
        pass
##        d = signer.add(self.GenerateHashBase())
##        def done(result):
##            dhnio.Dprint(14, 'dhnpacket.SignInThread.done')
##            self.Signature = result
##            self.SigningDefer.callback(self)
##        def fail(result):
##            dhnio.Dprint(1, 'dhnpacket.SignInThread.fail ERROR. starting self Sign')
##            self.Sign()
##            self.SigningDefer.callback(self)
##        d.addCallback(done)
##        d.addErrback(fail)

    def GenerateHashBase(self):
        sep = "-"
        stufftosum = self.Command + sep + self.OwnerID + sep + self.CreatorID + sep + self.PacketID + sep + self.Date + sep + self.Payload + sep + self.RemoteID
        return stufftosum

    def GenerateHash(self):
        return dhncrypto.Hash(self.GenerateHashBase())

    def GenerateSignature(self):
        hashcode = self.GenerateHash()
        ret = dhncrypto.Sign(hashcode)
        return ret

    def SignatureChecksOut(self):
        ConIdentity = contacts.getContact(self.CreatorID)
        if ConIdentity is None:
            dhnio.Dprint(1, "dhnpacket.SignatureChecksOut ERROR could not get Identity for " + self.CreatorID + " so returning False")
            return False
        Result = dhncrypto.Verify(ConIdentity, self.GenerateHash(), self.Signature)
        return Result

#  Valid should check every one of packet hearder fields:
#         1) that command is one of the legal commands
#         2) signature is good (which means the hashcode is good)
# Rest PREPRO:
#         2) all the number fields are just numbers
#         5) length is within legal limits
#         6) check that URL is a good URL
#         7) that DataOrParity is either "data" or "parity"
#         8) that creator is equal to owner or a scrubber for owner
#         etc

    def Ready(self):
        return self.Signature is not None

    def Valid(self):
        if not self.Ready():
            dhnio.Dprint(4, "dhnpacket.Valid WARNING packet is not ready yet " + str(self))
            return False

        if not commands.IsCommand(self.Command):
            dhnio.Dprint(1, "dhnpacket.Valid bad Command " + str(self.Command))
            return False

        if not self.SignatureChecksOut():
            dhnio.Dprint(1, "dhnpacket.Valid failed Signature")
            return False

        # if (self.BackupID()[0] != "F" and self.BackupID()[0] != "I"):       # first char of BackupID must be F or I
        #     dhnio.Dprint(1, "dhnpacket.Valid failed first char of BackupID")
        #     return(False)
        # dhnio.Dprint(9, "dhnpacket.Valid BackupID checks out ")
        # if (not misc.IsDigitsOnly(self.BackupID()[1:])):                  # rest of BackupID must be digits
        #     dhnio.Dprint(1, "dhnpacket.Valid failed first digits check of BackupID " + self.BackupID)
        #     return(False)
        # if (not misc.IsDigitsOnly(self.BlockNumber())):                   # BlockNumber is only digits
        #     dhnio.Dprint(1, "dhnpacket.Valid failed first digits check of BlockNumber " + self.BlockNumber)
        #     return(False)
        return True

    def BackupID(self):
        return packetid.BackupID(self.PacketID)

    def BlockNumber(self):
        return packetid.BlockNumber(self.PacketID)

    def DataOrParity(self):
        return packetid.DataOrParity(self.PacketID)

    def SupplierNumber(self):
        return packetid.SupplierNumber(self.PacketID)

    def Serialize(self):
        return misc.ObjectToString(self)

    def __len__(self):
        return len(self.Serialize())

def Unserialize(data):
    newobject = misc.StringToObject(data)
    if type(newobject) != types.InstanceType:
        dhnio.Dprint(1, "dhnpacket.Unserialize WARNING not an instance: " + str(newobject))
        return None
        #raise TypeError(str(data) + "\n is not a correct instance")
        #raise TypeError("Packet is not a correct instance")
    if newobject.__class__ != dhnpacket:
        dhnio.Dprint(1, "dhnpacket.Unserialize WARNING not a dhnpacket: " + str(newobject.__class__))
        #raise ValueError(str(data) + "\n is not a dhnpacket")
        return None
    return newobject

def MakePacket(Command, OwnerID, CreatorID, PacketID, Payload, RemoteID):
    result = dhnpacket(Command, OwnerID, CreatorID, PacketID, Payload, RemoteID)
    return result

def MakePacketInThread(CallBackFunc, Command, OwnerID, CreatorID, PacketID, Payload, RemoteID):
    d = threads.deferToThread(MakePacket, Command, OwnerID, CreatorID, PacketID, Payload, RemoteID)
    d.addCallback(CallBackFunc)

def MakePacketDeferred(Command, OwnerID, CreatorID, PacketID, Payload, RemoteID):
    return threads.deferToThread(MakePacket, Command, OwnerID, CreatorID, PacketID, Payload, RemoteID)



#nameurl.py

import re
import urllib
import urlparse

#  We assume DHN URLs are of the form ssh://host.name.com:port/fooobar.xml  maybe no foobar.xml
#  Tried urlparse and urlsplit and they move the location from the second to the 3rd
#     argument if you have "http" vs "ssh" or "tcp".  This seems like trouble.
def UrlParse(url):
    o = urlparse.urlparse(url)
    proto = o.scheme.strip()
    base = o.netloc.lstrip(' /')
    filename = o.path.lstrip(' /')
    if base == '':
        base = o.path.lstrip(' /')
        filename = ''

    if base.find('/') < 0:
        if base.find(':') < 0:
            host = base
            port = ''
        else:
            host, port = base.split(':', 1)
    else:
        host, tail = base.split('/', 1)
        if host.find(':') < 0:
            port = ''
        else:
            host, port = host.split(':', 1)

        if filename == '':
            filename = tail

    return proto.strip(), host.strip(), port.strip(), filename.strip()


def UrlMake(protocol, machine, port='', filename=''):
    url = protocol+'://'+machine
    if port != '':
        url += ':'+str(port)
    if filename != '':
        url += '/'+filename
    return url


legalchars="#.-_()ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"
legalset= set(legalchars)

def UrlFilename(url):
    if url is None:
        return None
    result = url.replace("://","###")
    result = result.replace("/","#")
    result = re.sub('(\:)(\d+)', '(#\g<2>#)', result)
    result = result.lower()
#    global legalset
#    # SECURITY  Test that all characters are legal ones
#    for x in result:
#        if x not in legalset:
#            raise Exception("nameurl.UrlFilename ERROR illegal character: \n" + url)
    return result

def FilenameUrl(filename):
    src = filename.strip().lower()
    if not src.startswith('http###'):
        return None
    src = re.sub('\(#(\d+)#\)', ':\g<1>', src)
    src = 'http://' + src[7:].replace('#', '/')
    return str(src)

def UrlFilenameHTML(url):
    global legalset
    s = url.replace('http://', '')
    o = ''
    for x in s:
        if x not in legalset or x == '.' or x == ':' or x == '#':
            o += '_'
        else:
            o += x
    return o

# deal with the identities
# for http://identity.datahaven.net/kinggeorge.xml
# should return "kinggeorge"
def GetName(url):
    if url in [None, 'None', ]:
        return ''
    #return url[url.rfind("/")+1:url.rfind(".")]
    return url[url.rfind("/")+1:-4]

def GetFileName(url):
    if url in [None, 'None', ]:
        return ''
    return url[url.rfind("/")+1:]

def Quote(s):
    return urllib.quote(s, '')

def UnQuote(s):
    return urllib.unquote(s)

def main():
#    url = 'http://identity.datahaven.net:565/sdfsdfsdf/veselin2.xml'
##    url = 'ssh://34.67.22.5: 5654 /gfgfg.sdfsdf/sdfsd'
##    url = 'q2q://d5wJMQRBYD72V6Zb5aZ1@work.offshore.ai'
##    print UrlParse(url)
#    print url
#    fn =  UrlFilename(url)
#    print fn
#    ur = FilenameUrl(fn)
#    print ur
##    print filenameurl
##    print FilenameUrl(filenameurl)
##    proto, machine, port, name = UrlParse(url)
##    url2 = UrlMake(proto, machine, 1234, name)
##    print url2
##    filenameurl2 = UrlFilename(url2)
##    print filenameurl2
##    print FilenameUrl(filenameurl2)
##    print UrlParse('q2q://d5wJMQRBYD72V6Zb5aZ1@work.offshore.ai')
    print GetName(str(None))

if __name__ == '__main__':
    main()













#!/usr/bin/python
#weblog.py

import sys
import urllib
import base64
from time import strftime

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in weblog.py')

from twisted.web import server, resource

myweblistener = None
default_level = 6
default_reload_timeout = 600
default_lines = 9999
maxlines = default_lines
numlines = 0
logtext = ''
lineindex = 0

header_html = '''<html><head>
<meta http-equiv="refresh" content="%(reload)s">
<title>Logs</title></head>
<body bgcolor="#FFFFFF" text="#000000" link="#0000FF" vlink="#0000FF">
<form action="" method="get">
<input size="4" name="level" value="%(level)s" />
<input size="4" name="reload" value="%(reload)s" />
<input size="10" name="lines" value="%(lines)s" />
<input type="submit" value="update" />
</form>
<a href="?reload=0.2&level=%(level)s&lines=%(lines)s">[1/5 sec.]</a>|
<a href="?reload=0.5&level=%(level)s&lines=%(lines)s">[1/2 sec.]</a>|
<a href="?reload=1&level=%(level)s&lines=%(lines)s">[1 sec.]</a>|
<a href="?reload=5&level=%(level)s&lines=%(lines)s">[5 sec.]</a>|
<a href="?reload=10&level=%(level)s&lines=%(lines)s">[10 sec.]</a>|
<a href="?reload=30&level=%(level)s&lines=%(lines)s">[30 sec.]</a>|
<a href="?reload=600&level=%(level)s&lines=%(lines)s">[600 sec.]</a>
<br>
<a href="?level=2&reload=%(reload)s&lines=%(lines)s">[debug2]</a>|
<a href="?level=4&reload=%(reload)s&lines=%(lines)s">[debug4]</a>|
<a href="?level=6&reload=%(reload)s&lines=%(lines)s">[debug6]</a>|
<a href="?level=8&reload=%(reload)s&lines=%(lines)s">[debug8]</a>|
<a href="?level=10&reload=%(reload)s&lines=%(lines)s">[debug10]</a>|
<a href="?level=12&reload=%(reload)s&lines=%(lines)s">[debug12]</a>|
<a href="?level=18&reload=%(reload)s&lines=%(lines)s">[debug18]</a>
<a href="?level=99&reload=%(reload)s&lines=%(lines)s">[debug99]</a>
<pre>
'''

level2color = {
    0: '',
    1: '',
    2: '',
    3: '',
    4: '#404040',
    5: '#404040',
    6: '#606060',
    7: '#606060',
    8: '#808080',
    9: '#808080',
    10: '#A0A0A0',
    11: '#A0A0A0',
    12: '#B0B0B0',
    13: '#B0B0B0',
    14: '#C0C0C0',
    15: '#C0C0C0',
    16: '#D0D0D0',
    17: '#D0D0D0',
    18: '#D0D0D0',
    19: '#D0D0D0',
    20: '#D0D0D0',
    21: '#D0D0D0',
    22: '#D0D0D0',
    23: '#D0D0D0',
    }

def log(level, s):
    global logtext
    global maxlines
    global numlines
    global lineindex
    global myweblistener
    if not myweblistener:
        return
    logtext += '%s|%s|%s|%s\n' % (str(lineindex), str(level), strftime('%H:%M:%S'), s.replace('\n', '#nl'))
    numlines += 1
    lineindex += 1
    while numlines > maxlines:
        logtext = logtext[logtext.find('\n')+1:]
        numlines -= 1

class LogPage(resource.Resource):
    def __init__(self,parent):
        self.parent = parent
        resource.Resource.__init__(self)

    def render(self, request):
        global logtext
        global maxlines
        DlevelS = request.args.get('level', [''])[0]
        try:
            DlevelV = int(DlevelS)
        except:
            DlevelV = default_level

        reloadS = request.args.get('reload', [''])[0]
        try:
            reloadV = float(reloadS)
        except:
            reloadV = default_reload_timeout

        linesS = request.args.get('lines', [''])[0]
        try:
            maxlines = int(linesS)
        except:
            maxlines = default_lines

        d = {'level': str(DlevelV), 'reload': str(reloadV), 'lines': str(maxlines)}
        out = header_html % d
        all_lines = logtext.splitlines()
        for lineindex in range(len(all_lines)-1, -1, -1):
            line = all_lines[lineindex] 
            t = line.split('|')
            try:
                #lineindex = int(t[0])
                level = int(t[1])
                timestr = t[2]
                s = ''.join(t[3:])
            except:
                #lineindex = 0
                level = 0
                timestr = ''
                s = line
            if level > DlevelV:
                continue
            
            s = s.strip().replace('#nl', '\n')
            s = ('%6d' % lineindex) + ' ' + timestr + (' ' * level) + ' ' + s
            
            textcolor = level2color.get(level, '')
            color = ''
            if s.find('BOGUS') > 0:
                color = 'purple'
                textcolor = 'white'
            if s.find('WARNING') > 0:
                color = 'blue'
                textcolor = 'white'
            if s.find('ERROR') > 0:
                color = 'red'
                textcolor = 'white'
            if s.find('NETERROR') > 0:
                color = 'gray'
                textcolor = 'white'
            if s.find('Exception:') > 0:
                color = 'red'
                textcolor = 'white'
            if s.find('] <<< ') > 0:
                color = '#ffffbb'
                textcolor = 'black'
            if s.find('] >>> ') > 0:
                color = '#bbffbb'
                textcolor = 'black'
                
            a = '%s'
            if color != '':
                a = ('<font color="%s" style="BACKGROUND-COLOR:%s">' % (textcolor, color)) + '%s</font>'
            elif textcolor != '':
                a = ('<font color="%s">' % textcolor) + '%s</font>'
            if level <= 4:
                a = '<b>%s</b>' % a

            try:
                out += a % str(s)
            except:
                try:
                    out += a % str(urllib.quote(s))
                except:
                    out += a % str(base64.encodestring(s))

        return out+'</pre></body></html>'

class RootResource(resource.Resource):
    def __init__(self):
        resource.Resource.__init__(self)
        logpage = LogPage(self)
        self.putChild('', logpage)

def init(port = 9999):
    global myweblistener
    if myweblistener:
        return
    root = RootResource()
    site = server.Site(root)
    try:
        myweblistener = reactor.listenTCP(port, site)
    except:
        pass

def shutdown():
    global myweblistener
    if myweblistener:
        myweblistener.stopListening()
        del myweblistener
        myweblistener = None

#------------------------------------------------------------------------------

if __name__ == "__main__":
    init(int(sys.argv[1]))
    reactor.run()











#!/usr/bin/python
#restore_monitor.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#
#manage currently restoring backups

import os
import sys
import time


import lib.dhnio as dhnio


import restore


_WorkingBackupIDs = {}
_WorkingRestoreProgress = {}
OnRestorePacketFunc = None
OnRestoreDoneFunc = None

#------------------------------------------------------------------------------ 

def init():
    dhnio.Dprint(4, 'restore_monitor.init')

def packet_in_callback(backupID, packet):
    dhnio.Dprint(4, 'restore_monitor.packet_in_callback ' + backupID)
    global _WorkingRestoreProgress
    global OnRestorePacketFunc
    
    SupplierNumber = packet.SupplierNumber()
    
    #want to count the data we restoring
    if SupplierNumber not in _WorkingRestoreProgress[backupID].keys():
        _WorkingRestoreProgress[backupID][SupplierNumber] = 0
    _WorkingRestoreProgress[backupID][SupplierNumber] += len(packet.Payload)

    if OnRestorePacketFunc is not None:
        OnRestorePacketFunc(backupID, SupplierNumber, packet)

def restore_done(x):
    dhnio.Dprint(2, 'restore_monitor.restore_done ' + str(x))
    global _WorkingBackupIDs
    global _WorkingRestoreProgress
    global OnRestoreDoneFunc
    
    backupID = x.split(' ')[0]
    _WorkingBackupIDs.pop(backupID, None)
    _WorkingRestoreProgress.pop(backupID, None)
    
    if OnRestoreDoneFunc is not None:
        OnRestoreDoneFunc(backupID, 'done')

def restore_failed(x, why='failed'):
    dhnio.Dprint(2, 'restore_monitor.restore_failed ' + str(x))
    global _WorkingBackupIDs
    global _WorkingRestoreProgress
    global OnRestoreDoneFunc

    backupID = x.split(' ')[0]
    _WorkingBackupIDs.pop(backupID, None)
    _WorkingRestoreProgress.pop(backupID, None)

    if OnRestoreDoneFunc is not None:
        OnRestoreDoneFunc(backupID, why)

def Start(backupID, outputFileName):
    dhnio.Dprint(6, 'restore_monitor.Start %s to %s' % (backupID, outputFileName))
    global _WorkingBackupIDs
    global _WorkingRestoreProgress
    if backupID in _WorkingBackupIDs.keys():
        return None
    try:
        outputFile = open(outputFileName, "wb")
    except:
        dhnio.Dprint(2, 'restore_monitor.Start ERROR can not open output file %s' % outputFileName)
        restore_failed(backupID, 'file_open_error')
        return None
       
    r = restore.restore(backupID, outputFile)
    r.MyDeferred.addCallback(restore_done)
    r.MyDeferred.addErrback(restore_failed)
    r.SetPacketInCallback(packet_in_callback)
    _WorkingBackupIDs[backupID] = r
    _WorkingRestoreProgress[backupID] = {}
    return r

def Abort(backupID):
    dhnio.Dprint(6, 'restore_monitor.Abort %s' % backupID)
    global _WorkingBackupIDs
    global _WorkingRestoreProgress
    if not backupID in _WorkingBackupIDs.keys():
        return False
    r = _WorkingBackupIDs[backupID]
    r.Abort()
    return True

def GetWorkingIDs():
    global _WorkingBackupIDs
    return _WorkingBackupIDs.keys()

def IsWorking(backupID):
    global _WorkingBackupIDs
    return backupID in _WorkingBackupIDs.keys()

def GetProgress(backupID):
    global _WorkingRestoreProgress
    return _WorkingRestoreProgress.get(backupID, {})










#!/usr/bin/python
#backupshedule.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#


import os
import sys
import time
import calendar


try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in backupshedule.py')

from twisted.internet.defer import Deferred


if __name__ == '__main__':
    dirpath = os.path.dirname(os.path.abspath(sys.argv[0]))
    sys.path.insert(0, os.path.abspath('datahaven'))
    sys.path.insert(0, os.path.abspath(os.path.join(dirpath, '..')))
    sys.path.insert(0, os.path.abspath(os.path.join(dirpath, '..', '..')))


import lib.dhnio as dhnio
import lib.misc as misc
import lib.dhnmath as dhnmath


import dobackup
import backup_db
import backups
import backup_monitor
import p2p_service
import webcontrol


_SheduledTasks = {}
_UsingGUI = False
_GuiStatusClearPageData = None
_GuiBackupUpdate = None


#------------------------------------------------------------------------------

def init(usingGui = False):
    global _UsingGUI
    _UsingGUI = usingGui
    dhnio.Dprint(4, 'backupshedule.init ')
    loop()


def shutdown():
    dhnio.Dprint(4, 'backupshedule.shutdown ')
    global _SheduledTasks
    for dirName, task in _SheduledTasks.items():
        try:
            task.cancel()
        except:
            dhnio.Dprint(1, 'backupshedule.shutdown ERROR can not stop task for ' + str(dirName))
            dhnio.DprintException()
        dhnio.Dprint(4, 'backupshedule.shutdown canceled: ' + str(dirName))


def loop():
    dhnio.Dprint(8, 'backupshedule.loop ')
    #debugWrite("in loop about to run at "+str(time.asctime(time.localtime(time.time()))))
    run()
    #debugWrite("in loop finished run, setting callLater")
    reactor.callLater(timeout(), loop)


#will check all Shedules each 60 minutes
def timeout():
    return 60.0*60.0


def run():
    global _SheduledTasks
    try:
        dhnio.Dprint(8, 'backupshedule.run ')
        for dirName in backup_db.GetBackupDirectories():
            dhnio.Dprint(8, 'backupshedule.run %s' % dirName)
            if backup_db.IsBackupRunning(dirName):
                dhnio.Dprint(8, 'backupshedule.run %s is running at the moment, skip.' % dirName)
                continue

            next_start = next(dirName)

            if next_start is None:
                # dhnio.Dprint(8, '  backupshedule.run next_start is None')
                continue

            if next_start < 0:
                continue

            now = time.time()
            delay = next_start - now
            dhnio.Dprint(8, '  backupshedule.run delay=%s next_start=[%s]' % (str(delay), str(time.asctime(time.localtime(next_start)))))
            if delay > 0 and delay < timeout():
                if _SheduledTasks.has_key(unicode(dirName)):
                    dhnio.Dprint(8, 'backupshedule.run cancel previous task')
                    try:
                        _SheduledTasks[unicode(dirName)].cancel()
                    except:
                        pass # may have already run or already been cancelled, nothing to cancel
                    del _SheduledTasks[unicode(dirName)]
                _SheduledTasks[unicode(dirName)] = reactor.callLater(delay, start_backup, dirName)
                # dhnio.Dprint(8, '  backupshedule.run will start after %s seconds' % (str(delay)))
                # dhnio.Dprint(8, '  backupshedule.run getTime()=' + str(_SheduledTasks[unicode(dirName)].getTime()))

    except:
        dhnio.DprintException()


def start_backup(dirName):
    global _SheduledTasks
    if _SheduledTasks.has_key(unicode(dirName)): # we're running now, no longer need the schedule
        del _SheduledTasks[unicode(dirName)]
    dhnio.Dprint(6, 'backupshedule.start_backup ' + str(dirName))

    if not backup_db.CheckDirectory(dirName):
        return

    bid = misc.NewBackupID()
    backups.AddBackupInProcess(bid)
    recursive_subfolders = backup_db.GetDirectorySubfoldersInclude(dirName)
    dir_size = dhnio.getDirectorySize(dirName, recursive_subfolders)
    result = Deferred()
    result.addCallback(backup_done)
    result.addErrback(backup_fail)
    dobackup.dobackup(bid, dirName, dir_size, recursive_subfolders, webcontrol.OnBackupDataPacketResult, result)

    global _UsingGUI
    global _GuiBackupUpdate
    if _UsingGUI:
        _GuiBackupUpdate()


def backup_done(backupID):
    dhnio.Dprint(4, 'backupshedule.backup_done ' + str(backupID))

    global _UsingGUI
    global _GuiBackupUpdate
    global _GuiStatusClearPageData

    if _UsingGUI:
        _GuiStatusClearPageData(backupID)

    aborted = False
    if backupID.endswith(' abort'):
        backupID = backupID[:-6]
        aborted = True

    backupDir = backup_db.GetDirectoryFromBackupId(backupID)
    if backupDir == "" or backupDir is None:
        dhnio.Dprint(6, 'backupshedule.backup_done  can not find %s  it in database' % backupID)
        return

    if aborted:
        backupDir = backup_db.GetDirectoryFromBackupId(backupID)
        backup_db.SetBackupStatus(backupDir, backupID, "stopped", "")
    else:
        backupDir = backup_db.GetDirectoryFromBackupId(backupID)
        backup_db.SetBackupStatus(backupDir, backupID, "done", str(time.time()))
    backup_db.Save()

    backups.RemoveBackupInProcess(backupID)
    backup_monitor.Restart()

    if aborted:
        if _UsingGUI:
            _GuiBackupUpdate()
    else:
        if _UsingGUI:
            _GuiBackupUpdate()


def backup_fail(backupID):
    dhnio.Dprint(4, 'backupshedule.backup_fail ' + str(backupID))
    global _UsingGUI
    global _GuiBackupUpdate
    global _GuiStatusClearPageData
    if _UsingGUI:
        _GuiStatusClearPageData(backupID)

    backupDir = backup_db.GetDirectoryFromBackupId(backupID)
    if backupDir == "" or backupDir is None:
        dhnio.Dprint(6, 'backupshedule.backup_fail  can not find %s  it in database' % str(backupID))
        return

    backup_db.SetBackupStatus(backupDir, backupID, "failed", "")
    backup_db.Save()
    backups.RemoveBackupInProcess(backupID)
    backup_monitor.Restart()

    if _UsingGUI:
        _GuiBackupUpdate()


def task(dirName=None):
    global _SheduledTasks
    if dirName is None:
        return _SheduledTasks
    return _SheduledTasks.get(unicode(dirName), None)


def next(dirName):
    global _SheduledTasks
#    lastRunId, lastRunSize, lastRunStatus, lastRunStart, lastRunFinish = backup_db.GetLastRunInfo(dirName)
#    schedule_type, schedule_time, schedule_interval, interval_details = backup_db.GetSchedule(dirName)
    schedule = backup_db.GetSchedule(dirName)
    if schedule is None:
        return None

#    dhnio.Dprint(8, 'backupshedule.next dirName=%s type=%s, time=%s, interval=%s, details=%s' % (str(dirName), schedule_type, schedule_time, schedule_interval, interval_details))
#    dhnio.Dprint(8, 'backupshedule.next %s %s' % (str(dirName), schedule))

    return schedule.next_time()
    
#    next_start = None
#    if schedule_type == 'none':
#        next_start = -1
#
#    elif schedule_type == 'hourly':
#        next_start = dhnmath.shedule_next_hourly(lastRunStart, schedule_interval)
#
#    elif schedule_type == 'daily':
#        next_start = dhnmath.shedule_next_daily(lastRunStart, schedule_interval, schedule_time)
#
#    elif schedule_type == 'weekly':
#        week_days = interval_details.split(' ')
#        week_day_numbers = []
#        week_day_names = list(calendar.day_name)
#        week_day_abbr = list(calendar.day_abbr)
#        for week_label in week_days:
#            try:
#                i = week_day_names.index(week_label)
#            except:
#                try:
#                    i = week_day_abbr.index(week_label)
#                except:
#                    continue
#            week_day_numbers.append(i)
#        next_start = dhnmath.shedule_next_weekly(lastRunStart, schedule_interval, schedule_time, week_day_numbers)
#
#    elif schedule_type == 'monthly':
#        months_labels = interval_details.split(' ')
#        months_numbers = []
#        months_names = list(calendar.month_name)
#        months_abbr = list(calendar.month_abbr)
#        for month_label in months_labels:
#            try:
#                i = months_names.index(month_label)
#            except:
#                try:
#                    i = months_abbr.index(month_label)
#                except:
#                    continue
#            months_numbers.append(i)
#        next_start = dhnmath.shedule_next_monthly(lastRunStart, schedule_interval, schedule_time, months_numbers)
#
#    return next_start


def debugWrite(debugtext): # useful when debugging this module, otherwise don't use (leave False)
    if False:
        debugFile = open("scheduledebug.txt","a")
        debugFile.write(debugtext+"\r\n")
        debugFile.close()
    else:
        dhnio.Dprint(8, debugtext)


#def types(k=None):
#    d = {'0': 'none',
#         '1': 'hourly',
#         '2': 'daily',
#         '3': 'weekly',
#         '4': 'monthly',
#         '5': 'continuously'}
#    if k is None:
#        return d
#    return d.get(k, d['0'])


#def labels():
#    return {'n': 'none',
#            'h': 'hourly',
#            'd': 'daily',
#            'w': 'weekly',
#            'm': 'monthly',
#            'c': 'continuously'}


#def default():
#    return {'type': 'none',
#            'interval': '1',
#            'time': '',
#            'details': '',}


#def format():
#    return '''
#Schedule compact format:
#[mode].[interval].[time].[details]
#
#mode:
#  n-none, h-hourly, d-daily, w-weekly, m-monthly
#  
#interval:
#  just a number - how often to restart the task, default is 1
#    
#time:
#  [hour]:[minute]
#  
#details:
#  for weeks: Mon Tue Wed Thu Fri Sat Sun
#  for months: Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec
#  
#some examples:
#  n.1..                   no schedule
#  hourly.3                each 3 hours
#  daily.4.10:15.          every 4th day at 10:15
#  w.1.3:00.MonSat         every Monday and Saturday at 3:00 in the night
#  weekly.4.18:45.MonTueWedThuFriSatSun
#                          every day in each 4th week in 18:45
#  m.5.12:34.JanJul        5th Jan and 5th July at 12:34
#'''


#def unpack(s):
#    try:
#        parts = s.split('.')
#        parts += ['']*(4-len(parts))
#        (sh_type, sh_interval, sh_time, sh_details) = parts[0:4]
#        sh_type = sh_type.lower()
#        if sh_type not in ['n', 'none', 'h', 'hourly']:
#            sh_interval = int(sh_interval)
#            time.strptime(sh_time, '%H:%M')
#    except:
#        dhnio.Dprint(2, 'backupshedule.unpack WARNING incorrect shedule '+s)
#        dhnio.DprintException()
#        return None
#    if sh_type in labels().keys():
#        sh_type = labels()[sh_type]
#    if sh_type not in labels().values():
#        dhnio.Dprint(2, 'backupshedule.unpack WARNING incorrect shedule '+s)
#        return None
#    sh_details_new = ''
#    for i in range(len(sh_details)/3):
#        label = sh_details[i*3:i*3+3]
#        if sh_type == 'weekly' and not label in calendar.day_abbr:
#            dhnio.Dprint(2, 'backupshedule.unpack WARNING incorrect shedule '+s)
#            return None
#        if sh_type == 'monthly' and not label in calendar.month_abbr:
#            dhnio.Dprint(2, 'backupshedule.unpack WARNING incorrect shedule '+s)
#            return None
#        sh_details_new += label + ' '
#    return {'type': sh_type,
#            'interval': str(sh_interval),
#            'time': str(sh_time),
#            'details': sh_details_new.strip(),}


#def split(t):
#    typ = str(t[0])
#    if typ in types().keys():
#        typ = types()[typ]
#    if typ not in types().values():
#        dhnio.Dprint(2, 'backupshedule.split WARNING incorrect shedule '+str(t))
#        return default()
#    return {
#        'type':         typ,
#        'time':         str(t[1]),
#        'interval':     str(t[2]),
#        'details':      str(t[3]), }


#-------------------------------------------------------------------------------


def main():
    nt = dhnmath.shedule_next_daily(time.time()-60*60*24*2, 4, '12:00')
    print 'daily', time.asctime(time.localtime(nt))
    nw = dhnmath.shedule_next_weekly(time.time(),1,'12:00', [0,])
    print 'weekly', time.asctime(time.localtime(nw))
    nm = dhnmath.shedule_next_monthly(time.time(),1,'12:00', [1,])
    print 'monthly', time.asctime(time.localtime(nm))
    print unpack('weekly.4.18:45.MonTueWedThuFriSatSun')
    print unpack('h.1')


if __name__ == '__main__':
    dhnio.SetDebug(12)
    main()





#!/usr/bin/env python
#list_files_orator.py
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#

import os
import sys

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in list_files_orator.py')
from twisted.internet.defer import maybeDeferred
from twisted.internet.task import LoopingCall


from lib.automat import Automat
import lib.dhnio as dhnio
import lib.contacts as contacts


import backup_monitor
import p2p_connector
import lib.automats as automats

import backups 
import p2p_service
import contacts_status


_ListFilesOrator = None
_RequestedListFilesPacketIDs = set()
_RequestedListFilesCounter = 0

#------------------------------------------------------------------------------

def A(event=None, arg=None):
    global _ListFilesOrator
    if _ListFilesOrator is None:
        _ListFilesOrator = ListFilesOrator('list_files_orator', 'NO_FILES', 4)
    if event is not None:
        _ListFilesOrator.automat(event, arg)
    return _ListFilesOrator


class ListFilesOrator(Automat):
    timers = {'timer-1min':  (60, ['REMOTE_FILES']), }

    def state_changed(self, oldstate, newstate):
        #automats.set_global_state('ORATOR ' + newstate)
        backup_monitor.A('list_files_orator.state', newstate)

    def A(self, event, arg):
        #---NO_FILES---
        if self.state == 'NO_FILES':
            if event == 'need-files':
                self.doReadLocalFiles()
                self.state = 'LOCAL_FILES'
            elif event == 'init':
                pass
        #---LOCAL_FILES---
        elif self.state == 'LOCAL_FILES':
            if event == 'local-files-done' and p2p_connector.A().state is 'CONNECTED':
                
                self.doRequestRemoteFiles()
                self.state = 'REMOTE_FILES'
            elif event == 'local-files-done' and p2p_connector.A().state is not 'CONNECTED':
                self.state = 'NO_FILES'
        #---REMOTE_FILES---
        elif self.state == 'REMOTE_FILES':
            if ( event == 'timer-1min' and self.isSomeListFilesReceived() ) or ( event == 'inbox-files' and self.isAllListFilesReceived() ):
                self.state = 'SAW_FILES'
            elif event == 'timer-1min' and not self.isSomeListFilesReceived():
                self.state = 'NO_FILES'
        #---SAW_FILES---
        elif self.state == 'SAW_FILES':
            if event == 'need-files':
                self.doReadLocalFiles()
                self.state = 'LOCAL_FILES'

    def isAllListFilesReceived(self):
        global _RequestedListFilesPacketIDs
        dhnio.Dprint(6, 'list_files_orator.isAllListFilesReceived need %d more' % len(_RequestedListFilesPacketIDs))
        return len(_RequestedListFilesPacketIDs) == 0

    def isSomeListFilesReceived(self):
        global _RequestedListFilesCounter
        dhnio.Dprint(6, 'list_files_orator.isSomeListFilesReceived %d list files was received' % _RequestedListFilesCounter)
        return _RequestedListFilesCounter > 0

    def doReadLocalFiles(self):
        maybeDeferred(backups.ReadLocalFiles).addBoth(
            lambda x: self.automat('local-files-done'))
    
    def doRequestRemoteFiles(self):
        global _RequestedListFilesCounter
        global _RequestedListFilesPacketIDs
        _RequestedListFilesCounter = 0
        _RequestedListFilesPacketIDs.clear()
        for idurl in contacts.getSupplierIDs():
            if contacts_status.isOnline(idurl):
                p2p_service.RequestListFiles(idurl)
                _RequestedListFilesPacketIDs.add(idurl)
                
#    def doReadSavedRemoteListFiles(self):
#        backups.ReadLatestRawListFiles()
    

def IncommingListFiles(supplierNum, packet, listFileText):
    global _RequestedListFilesPacketIDs
    global _RequestedListFilesCounter
    _RequestedListFilesCounter += 1
    _RequestedListFilesPacketIDs.discard(packet.OwnerID)
    A('inbox-files', (supplierNum, packet, listFileText))
    
    





#!/usr/bin/python
#sweeptemp.py
#
#
#    Copyright DataHaven.NET LTD. of Anguilla, 2006-2012
#    Use of this software constitutes acceptance of the Terms of Use
#      http://datahaven.net/terms_of_use.html
#    All rights reserved.
#
#
#Temp directory:
#earlier today I had over 30000, 3GB of DHN related files in temp.
#Something to keep an eye on this, clean up after ourselves over time.

import os
import sys
import time

try:
    from twisted.internet import reactor
except:
    sys.exit('Error initializing twisted.internet.reactor in sweeptemp.py')

from twisted.internet.defer import Deferred


import datahaven.lib.dhnio as dhnio
import datahaven.lib.settings as settings


_LoopObject = None
#will scan Temp folder each N minutes
_DefaultDelay = 60 * 60 # 1 hour sufficient, maybe even just once a day, temp directory could be 1000's of files, don't want to scan often

#will remove only files with listed below extensions and intervals
_ExtensionsToDelete = {
'.controloutbox':60*60*24*1, # hold onto outbox files 1 day so we can handle resends if contact offline
'.dhn-tcp-in':   60*10,      # 10 minutes for incoming files
'.dhn-http-in':  60*30,
'.dhn-q2q-in':   60*10,      # 10 minutes for incoming files
'.dhn-cspace-in':60*10,      # 10 minutes for incoming files
'.dhn-version':  60*10,
'.dhn-info':     60*10,
'-dhn.bat':      60*10,
#'.dhn-tcp-out':  60*60*24*1, # hold on 1 day to outgoing tcp
'.propagate':    60*10,      # propagate happens often enough, 10 minutes should be enough
'.backup':       60*10,
'-data':         60*60*24*1, # hold onto data for rebuilding for a day, can rebuild parity as needed
'-parity':       60*60,      # hold onto parity for an hour
}
#we do not want to remove some files. keep it in this list.
#just filenames here
_ExculdedFiles = set()

def exclude(filename, exclude_off=False):
    global _ExculdedFiles
    fn = os.path.basename(filename)
    if not exclude_off:
        _ExculdedFiles.add(fn)
    else:
        _ExculdedFiles.discard(fn)

def delete(filepath):
    dhnio.Dprint(8, 'sweeptemp.delete ' + os.path.basename(str(filepath)))
    try:
        os.remove(filepath)
    except:
        dhnio.DprintException()

def run():
    global _ExtensionsToDelete
    global _ExculdedFiles
    dhnio.Dprint(6, 'sweeptemp.run')

    deleteCount = 0
    tempdir = os.path.abspath(settings.TempDir())
    for filename in os.listdir(tempdir):
        if filename in _ExculdedFiles:
            continue
        filepath = os.path.abspath(os.path.join(tempdir, filename))

#        if not os.path.isfile(filepath):
#            continue
#        if not os.access(filepath, os.R_OK):
#            continue
#        if not os.access(filepath, os.W_OK):
#            continue
#        extension_interest = False

        for ext in _ExtensionsToDelete.keys():
            if filename.lower().endswith(ext):
                dhnio.Dprint(10, 'sweeptemp.run   %s' % filename)
                try:
                    tm = os.path.getatime(filepath)
                    if (time.time() - tm) > _ExtensionsToDelete[ext]:
                        delete(filepath)
                        deleteCount += 1
                        if deleteCount > 100:
                            return True
                except:
                    dhnio.DprintException()

                break

    return False


def loop():
    global _LoopObject
    #reactor.callInThread(run) # if we run in another thread, if we're deleting stuff and looking to shutdown, it won't let us exit
    if run(): # if we delete found 100 files to delete, potentially more
        _LoopObject = reactor.callLater(30, loop)
    else:
        _LoopObject = reactor.callLater(_DefaultDelay, loop)

def init():
    dhnio.Dprint(4, 'sweeptemp.init')
    loop()


if __name__ == '__main__':
    dhnio.SetDebug(18)
    init()
    reactor.run()
    
    